@@ -3,9 +3,9 @@
 
 #include "amici/defines.h" // necessary for realtype
 
-#include <exception>
 #include <array>
 #include <cstdarg>
+#include <exception>
 
 namespace amici {
 
@@ -15,11 +15,9 @@ namespace amici {
  * Has a printf style interface to allow easy generation of error messages
  */
 class AmiException : public std::exception {
-public:
+  public:
     /**
-     * @brief Constructor with printf style interface
-     * @param fmt error message with printf format
-     * @param ... printf formatting variables
+     * @brief Default ctor.
      */
     AmiException();
 
@@ -34,13 +32,13 @@ public:
      * @brief Override of default error message function
      * @return msg error message
      */
-    const char* what() const noexcept override;
+    char const* what() const noexcept override;
 
     /**
      * @brief Returns the stored backtrace
      * @return trace backtrace
      */
-    const char *getBacktrace() const;
+    char const* getBacktrace() const;
 
     /**
      * @brief Stores the current backtrace
@@ -54,42 +52,39 @@ public:
      * @param fmt error message with printf format
      * @param argptr pointer to variadic argument list
      */
-    void storeMessage(const char *fmt, va_list argptr);
+    void storeMessage(char const* fmt, va_list argptr);
 
-private:
+  private:
     std::array<char, 500> msg_;
     std::array<char, 500> trace_;
 };
 
-
 /**
  * @brief cvode exception handler class
  */
-class CvodeException : public AmiException  {
-public:
+class CvodeException : public AmiException {
+  public:
     /**
      * @brief Constructor
      * @param error_code error code returned by cvode function
      * @param function cvode function name
      */
-    CvodeException(int error_code, const char *function);
+    CvodeException(int error_code, char const* function);
 };
 
-
 /**
  * @brief ida exception handler class
  */
-class IDAException : public AmiException  {
-public:
+class IDAException : public AmiException {
+  public:
     /**
      * @brief Constructor
      * @param error_code error code returned by ida function
      * @param function ida function name
      */
-    IDAException(int error_code, const char *function);
+    IDAException(int error_code, char const* function);
 };
 
-
 /**
  * @brief Integration failure exception for the forward problem
  *
@@ -97,7 +92,7 @@ public:
  * for this exception we can assume that we can recover from the exception
  * and return a solution struct to the user
  */
-class IntegrationFailure : public AmiException  {
+class IntegrationFailure : public AmiException {
   public:
     /**
      * @brief Constructor
@@ -113,7 +108,6 @@ class IntegrationFailure : public AmiException  {
     realtype time;
 };
 
-
 /**
  * @brief Integration failure exception for the backward problem
  *
@@ -121,7 +115,7 @@ class IntegrationFailure : public AmiException  {
  * for this exception we can assume that we can recover from the exception
  * and return a solution struct to the user
  */
-class IntegrationFailureB : public AmiException  {
+class IntegrationFailureB : public AmiException {
   public:
     /**
      * @brief Constructor
@@ -137,7 +131,6 @@ class IntegrationFailureB : public AmiException  {
     realtype time;
 };
 
-
 /**
  * @brief Setup failure exception
  *
@@ -153,10 +146,8 @@ class SetupFailure : public AmiException {
      * @param ... printf formatting variables
      */
     explicit SetupFailure(char const* fmt, ...);
-
 };
 
-
 /**
  * @brief Newton failure exception
  *
@@ -165,13 +156,13 @@ class SetupFailure : public AmiException {
  * recover from the exception and return a solution struct to the user
  */
 class NewtonFailure : public AmiException {
-public:
+  public:
     /**
      * @brief Constructor, simply calls AmiException constructor
      * @param function name of the function in which the error occurred
      * @param code error code
      */
-    NewtonFailure(int code, const char *function);
+    NewtonFailure(int code, char const* function);
 
     /** error code returned by solver */
     int error_code;
