@@ -5,7 +5,7 @@
 #include "amici/vector.h"
 #include "amici/model.h"
 #include "amici/misc.h"
-#include "amici/sundials_matrix_wrapper.h"
+#include <amici/amici.h>
 
 #include <sundials/sundials_direct.h>
 #include <vector>
@@ -18,23 +18,6 @@ class Solver;
 class SteadystateProblem;
 class FinalStateStorer;
 
-/**
- * @brief implements an exchange format to store and transfer the state of a simulation at a
- * specific timepoint.
- */
-struct SimulationState{
-    /** timepoint */
-    realtype t;
-    /** state variables */
-    AmiVector x;
-    /** state variables */
-    AmiVector dx;
-    /** state variable sensitivity */
-    AmiVectorArray sx;
-    /** state of the model that was used for simulation */
-    ModelState state;
-};
-
 
 /**
  * @brief The ForwardProblem class groups all functions for solving the
@@ -221,7 +204,7 @@ class ForwardProblem {
      * @return index
      */
     int getEventCounter() const {
-        return static_cast<int>(event_states_.size() - 1);
+        return gsl::narrow<int>(event_states_.size()) - 1;
     }
 
     /**
@@ -229,7 +212,7 @@ class ForwardProblem {
      * @return index
      */
     int getRootCounter() const {
-        return static_cast<int>(discs_.size() - 1);
+        return gsl::narrow<int>(discs_.size()) - 1;
     }
 
     /**
